@@ -1,7 +1,8 @@
-(def ks-version "0.7.2")
-(def tk-version "0.5.1")
+(def ks-version "1.0.0")
+(def tk-version "1.1.0")
+(def tk-jetty-version "1.2.0")
 
-(defproject puppetlabs/http-client "0.4.0"
+(defproject puppetlabs/http-client "0.4.4"
   :description "HTTP client wrapper"
   :license {:name "Apache License, Version 2.0"
             :url "http://www.apache.org/licenses/LICENSE-2.0.html"}
@@ -11,15 +12,16 @@
   ;; requires lein 2.2.0+.
   :pedantic? :abort
 
-  :dependencies [[org.clojure/clojure "1.5.1"]
-                 [puppetlabs/certificate-authority "0.6.0"]
+  :dependencies [[org.clojure/clojure "1.6.0"]
+                 [puppetlabs/ssl-utils "0.8.0"]
                  [org.clojure/tools.logging "0.2.6"]
                  [puppetlabs/kitchensink ~ks-version]
                  [org.slf4j/slf4j-api "1.7.6"]
                  [org.apache.httpcomponents/httpasyncclient "4.0.2"]
                  [org.apache.httpcomponents/httpcore "4.3.2"]
                  [commons-io "2.1"]
-                 [prismatic/schema "0.2.2"]]
+                 [prismatic/schema "0.4.0"]
+                 [prismatic/plumbing "0.4.2"]]
 
   :source-paths ["src/clj"]
   :java-source-paths ["src/java"]
@@ -33,9 +35,12 @@
   :profiles {:dev {:dependencies [[puppetlabs/kitchensink ~ks-version :classifier "test"]
                                   [puppetlabs/trapperkeeper ~tk-version]
                                   [puppetlabs/trapperkeeper ~tk-version :classifier "test"]
-                                  [puppetlabs/trapperkeeper-webserver-jetty9 "0.9.0"]
-                                  [spyscope "0.1.4"]]
-                   :injections [(require 'spyscope.core)]}
+                                  [puppetlabs/trapperkeeper-webserver-jetty9 ~tk-jetty-version]
+                                  [puppetlabs/trapperkeeper-webserver-jetty9 ~tk-jetty-version :classifier "test"]
+                                  [spyscope "0.1.4" :exclusions [clj-time]]]
+                   :injections [(require 'spyscope.core)]
+                   ;; TK-143, enable SSLv3 for unit tests that exercise SSLv3
+                   :jvm-opts ["-Djava.security.properties=./dev-resources/java.security"]}
              :sources-jar {:java-source-paths ^:replace []
                            :jar-exclusions ^:replace []
                            :source-paths ^:replace ["src/clj" "src/java"]}}
@@ -48,4 +53,4 @@
   :lein-release {:scm :git
                  :deploy-via :lein-deploy}
 
-  :plugins [[lein-release "1.0.5"]])
+  :plugins [[lein-release "1.0.5" :exclusions [org.clojure/clojure]]])
