@@ -1,7 +1,7 @@
-(def tk-version "1.0.1")
-(def tk-jetty-version "1.1.1")
-(def ks-version "1.0.0")
-(def ps-version "2.0.0")
+(def tk-version "1.1.1")
+(def tk-jetty-version "1.3.1")
+(def ks-version "1.1.0")
+(def ps-version "2.1.0")
 
 (defn deploy-info
   [url]
@@ -17,37 +17,33 @@
                  [puppetlabs/trapperkeeper ~tk-version]
                  [puppetlabs/kitchensink ~ks-version]
                  [puppetlabs/ssl-utils "0.8.0"]
-                 [puppetlabs/http-client "0.4.2"]
                  [puppetlabs/dujour-version-check "0.1.2" :exclusions [org.clojure/tools.logging]]
-                 [org.jruby/jruby-core "1.7.18"
-                  :exclusions
-                  [com.github.jnr/jffi com.github.jnr/jnr-x86asm com.github.jnr/jnr-ffi
-                   org.ow2.asm/asm org.ow2.asm/asm-commons org.ow2.asm/asm-analysis
-                   org.ow2.asm/asm-util com.github.jnr/jnr-constants]]
-                 ;; NOTE: the JRuby poms (as of 1.7.18) had some conflicting transitive dependencies
-                 ;; which necessitated the above exclusions and the following explicit versions of
-                 ;; those transitive deps.  We should check to see if this issue is resolved
-                 ;; in 1.7.19.
-                 [com.github.jnr/jffi "1.2.7"]
-                 [com.github.jnr/jffi "1.2.7" :classifier "native"]
+                 [puppetlabs/http-client "0.4.4"]
+                 [org.jruby/jruby-core "1.7.20"
+                  :exclusions [com.github.jnr/jffi com.github.jnr/jnr-x86asm]]
+                 ;; jffi and jnr-x86asm are explicit dependencies because,
+                 ;; in JRuby's poms, they are defined using version ranges,
+                 ;; and :pedantic? :abort won't tolerate this.
+                 [com.github.jnr/jffi "1.2.9"]
+                 [com.github.jnr/jffi "1.2.9" :classifier "native"]
                  [com.github.jnr/jnr-x86asm "1.0.2"]
-                 [com.github.jnr/jnr-ffi "2.0.1"]
-                 [com.github.jnr/jnr-constants "0.8.6"]
                  ;; NOTE: jruby-stdlib packages some unexpected things inside
                  ;; of its jar; please read the detailed notes above the
                  ;; 'uberjar-exclusions' example toward the end of this file.
-                 [org.jruby/jruby-stdlib "1.7.18"]
-                 [clj-time "0.5.1" :exclusions [joda-time]]
-                 [compojure "1.1.8" :exclusions [org.clojure/tools.macro]]
+                 [org.jruby/jruby-stdlib "1.7.20"]
+                 [org.clojure/data.json "0.2.3"]
+                 [org.clojure/tools.macro "0.1.5"]
+                 [joda-time "2.5"]
+                 [clj-time "0.6.0"]
                  [liberator "0.12.0"]
+                 [puppetlabs/comidi "0.1.1"]
                  [me.raynes/fs "1.4.5"]
-                 [prismatic/schema "0.2.2"]
+                 [prismatic/schema "0.4.0"]
                  [commons-lang "2.6"]
                  [commons-io "2.4"]
                  [commons-codec "1.9"]
                  [clj-yaml "0.4.0" :exclusions [org.yaml/snakeyaml]]
                  [slingshot "0.10.3"]
-                 [ring/ring-codec "1.0.0" :exclusions [commons-codec]]
                  [cheshire "5.3.1"]
                  [trptcolin/versioneer "0.1.0"]]
 
@@ -98,11 +94,12 @@
              :ezbake {:dependencies ^:replace [[puppetlabs/puppetserver ~ps-version]
                                                [puppetlabs/trapperkeeper-webserver-jetty9 ~tk-jetty-version]
                                                [org.clojure/tools.nrepl "0.2.3"]]
-                      :plugins [[puppetlabs/lein-ezbake "0.3.2"]]
+                      :plugins [[puppetlabs/lein-ezbake "0.3.6"]]
                       :name "puppetserver"}
              :uberjar {:aot [puppetlabs.trapperkeeper.main]
                        :dependencies [[puppetlabs/trapperkeeper-webserver-jetty9 ~tk-jetty-version]]}
-             :ci {:plugins [[lein-pprint "1.1.1"]]}}
+             :ci {:plugins [[lein-pprint "1.1.1"]]}
+             :voom {:plugins [[lein-voom "0.1.0-20150115_230705-gd96d771" :exclusions [org.clojure/clojure]]]}}
 
   :test-selectors {:integration :integration
                    :unit (complement :integration)}
@@ -112,7 +109,7 @@
             "irb" ["trampoline" "run" "-m" "puppetlabs.puppetserver.cli.irb"]}
 
   ; tests use a lot of PermGen (jruby instances)
-  :jvm-opts ["-XX:MaxPermSize=256m"]
+  :jvm-opts ["-XX:MaxPermSize=256m" "-Xmx1024M"]
 
   :repl-options {:init-ns user}
 
