@@ -4,10 +4,11 @@
             [puppetlabs.services.protocols.jruby-puppet :as jruby-puppet]
             [puppetlabs.puppetserver.liberator-utils :as liberator-utils]
             [schema.core :as schema]
-            [compojure.core :as compojure]
-            [compojure.route :as route]
             [liberator.core :refer [defresource]]
-            [liberator.dev :as liberator-dev]))
+    ;[liberator.dev :as liberator-dev]
+            [puppetlabs.comidi :as comidi]
+            [clojure.tools.logging :as log]
+            [puppetlabs.puppetserver.ring.middleware.params :as pl-ring-params]))
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -22,7 +23,7 @@
 ;;; Liberator resources
 
 (defresource environment-cache-resource
-  [jruby-service]
+  [jruby-service env-name]
   :allowed-methods [:delete]
 
   ;; If you need to define :available-media-types, see comment below.
@@ -45,7 +46,9 @@
 
   :delete!
   (fn [context]
-    (jruby-puppet/mark-all-environments-expired! jruby-service)))
+    (if env-name
+      (jruby-puppet/mark-environment-expired! jruby-service env-name)
+      (jruby-puppet/mark-all-environments-expired! jruby-service))))
 
 (defresource jruby-pool-resource
   [jruby-service]
@@ -81,17 +84,18 @@
 (defn v1-routes
   [jruby-service]
   "Routes for v1 of the Puppet Admin HTTP API."
-  (compojure/routes
-    (compojure/ANY "/environment-cache" []
-      (environment-cache-resource jruby-service))
-    (compojure/ANY "/jruby-pool" []
+  (comidi/routes
+    (comidi/ANY "/environment-cache" request
+      (environment-cache-resource jruby-service
+        (get-in request [:query-params "environment"])))
+    (comidi/ANY "/jruby-pool" []
        (jruby-pool-resource jruby-service))))
 
 (defn versioned-routes
   [jruby-service]
-  (compojure/routes
-    (compojure/context "/v1" [] (v1-routes jruby-service))
-    (route/not-found "Not Found")))
+  (comidi/routes
+    (comidi/context "/v1" (v1-routes jruby-service))
+    (comidi/not-found "Not Found")))
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -111,10 +115,10 @@
   "Returns the ring handler for the Puppet Admin API."
   [path :- schema/Str
    settings :- PuppetAdminSettings
-   jruby-service :- jruby-puppet/JRubyPuppetService]
-  (-> (compojure/context
-        path
-        []
-        (versioned-routes jruby-service))
+   jruby-service :- (schema/protocol jruby-puppet/JRubyPuppetService)]
+  (-> (versioned-routes jruby-service)
+      (#(comidi/context path %))
+      (comidi/routes->handler)
       ;(liberator-dev/wrap-trace :header)           ; very useful for debugging!
-      (ringutils/wrap-with-cert-whitelist-check settings)))
+      (ringutils/wrap-with-cert-whitelist-check settings)
+      pl-ring-params/wrap-params))
